@@ -21,7 +21,6 @@
 #include <stdint.h>
 #include <stdlib.h>
 #include <sys/types.h>
-#include <malloc.h>
 #include <string.h>
 #include <stdio.h>
 #include <assert.h>
@@ -156,7 +155,7 @@ retvalue checksums_init(/*@out@*/struct checksums **checksums_p, char *hashes[cs
 			free(hashes[type]);
 		return RET_ERROR_OOM;
 	}
-	memset(n, 0, sizeof(struct checksums));
+	setzero(struct checksums, n);
 	d = n->representation;
 
 	for (type = cs_firstEXTENDED ; type < cs_hashCOUNT ; type++) {
@@ -223,7 +222,7 @@ retvalue checksums_initialize(struct checksums **checksums_p, const struct hash_
 	if (FAILEDTOALLOC(n))
 		return RET_ERROR_OOM;
 
-	memset(n, 0, sizeof(struct checksums));
+	setzero(struct checksums, n);
 	d = n->representation;
 
 	for (type = cs_firstEXTENDED ; type < cs_hashCOUNT ; type++) {
@@ -277,7 +276,7 @@ retvalue checksums_parse(struct checksums **checksums_p, const char *combinedche
 	n = malloc(sizeof(struct checksums) + len + 1);
 	if (FAILEDTOALLOC(n))
 		return RET_ERROR_OOM;
-	memset(n, 0, sizeof(struct checksums));
+	setzero(struct checksums, n);
 	d = n->representation;
 	while (*p == ':') {
 
@@ -504,7 +503,7 @@ retvalue checksums_combine(struct checksums **checksums_p, const struct checksum
 	n = malloc(sizeof(struct checksums)+ len + 1);
 	if (FAILEDTOALLOC(n))
 		return RET_ERROR_OOM;
-	memset(n, 0, sizeof(struct checksums));
+	setzero(struct checksums, n);
 	o = old->representation;
 	b = by->representation;
 	d = n->representation;
@@ -1141,7 +1140,6 @@ static const char tab[16] = {'0', '1', '2', '3', '4', '5', '6', '7',
                              '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};
 
 retvalue checksums_from_context(struct checksums **out, struct checksumscontext *context) {
-#define MD5_DIGEST_SIZE 16
 	unsigned char md5buffer[MD5_DIGEST_SIZE], sha1buffer[SHA1_DIGEST_SIZE],
 		      sha256buffer[SHA256_DIGEST_SIZE];
 	char *d;
@@ -1152,7 +1150,7 @@ retvalue checksums_from_context(struct checksums **out, struct checksumscontext
 			+ 2*SHA1_DIGEST_SIZE + 2*SHA256_DIGEST_SIZE + 30);
 	if (FAILEDTOALLOC(n))
 		return RET_ERROR_OOM;
-	memset(n, 0, sizeof(struct checksums));
+	setzero(struct checksums, n);
 	d = n->representation;
 	*(d++) = ':';
 	*(d++) = '1';
