@@ -32,7 +32,7 @@
 
 struct downloaditem {
 	/*@dependent@*//*@null@*/struct downloaditem *parent;
-	/*@null@*/struct downloaditem *left,*right;
+	/*@null@*/struct downloaditem *left, *right;
 	char *filekey;
 	struct checksums *checksums;
 	bool done;
@@ -43,11 +43,11 @@ retvalue downloadcache_initialize(enum spacecheckmode mode, off_t reserveddb, of
 	struct downloadcache *cache;
 	retvalue r;
 
-	cache = calloc(1, sizeof(struct downloadcache));
-	if( cache == NULL )
+	cache = zNEW(struct downloadcache);
+	if (FAILEDTOALLOC(cache))
 		return RET_ERROR_OOM;
 	r = space_prepare(&cache->devices, mode, reserveddb, reservedother);
-	if( RET_WAS_ERROR(r) ) {
+	if (RET_WAS_ERROR(r)) {
 		free(cache);
 		return r;
 	}
@@ -57,7 +57,7 @@ retvalue downloadcache_initialize(enum spacecheckmode mode, off_t reserveddb, of
 
 /* free all memory */
 static void freeitem(/*@null@*//*@only@*/struct downloaditem *item) {
-	if( item == NULL )
+	if (item == NULL)
 		return;
 	freeitem(item->left);
 	freeitem(item->right);
@@ -67,7 +67,7 @@ static void freeitem(/*@null@*//*@only@*/struct downloaditem *item) {
 }
 
 retvalue downloadcache_free(struct downloadcache *download) {
-	if( download == NULL )
+	if (download == NULL)
 		return RET_NOTHING;
 
 	freeitem(download->items);
@@ -79,53 +79,53 @@ retvalue downloadcache_free(struct downloadcache *download) {
 static retvalue downloaditem_callback(enum queue_action action, void *privdata, void *privdata2, const char *uri, const char *gotfilename, const char *wantedfilename, /*@null@*/const struct checksums *checksums, const char *method) {
 	struct downloaditem *d = privdata;
 	struct downloadcache *cache = privdata2;
-	struct database *database = cache->database;
 	struct checksums *read_checksums = NULL;
 	retvalue r;
 	bool improves;
 
-	if( action != qa_got )
+	if (action != qa_got)
 		// TODO: instead store in downloaditem?
 		return RET_ERROR;
 
 	/* if the file is somewhere else, copy it: */
-	if( strcmp(gotfilename, wantedfilename) != 0 ) {
-		if( verbose > 1 )
+	if (strcmp(gotfilename, wantedfilename) != 0) {
+		if (verbose > 1)
 			fprintf(stderr,
 "Linking file '%s' to '%s'...\n", gotfilename, wantedfilename);
 		r = checksums_linkorcopyfile(wantedfilename, gotfilename,
 				&read_checksums);
-		if( r == RET_NOTHING ) {
-			fprintf(stderr, "Cannot open '%s', obtained from '%s' method.\n",
+		if (r == RET_NOTHING) {
+			fprintf(stderr,
+"Cannot open '%s', obtained from '%s' method.\n",
 					gotfilename, method);
 			r = RET_ERROR_MISSING;
 		}
-		if( RET_WAS_ERROR(r) ) {
+		if (RET_WAS_ERROR(r)) {
 			// TODO: instead store in downloaditem?
 			return r;
 		}
-		if( read_checksums != NULL )
+		if (read_checksums != NULL)
 			checksums = read_checksums;
 	}
 
-	if( checksums == NULL || !checksums_iscomplete(checksums) ) {
+	if (checksums == NULL || !checksums_iscomplete(checksums)) {
 		assert(read_checksums == NULL);
 		r = checksums_read(wantedfilename, &read_checksums);
-		if( r == RET_NOTHING ) {
+		if (r == RET_NOTHING) {
 			fprintf(stderr,
 "Cannot open '%s', though '%s' method claims to have put it there!\n",
 					wantedfilename, method);
 			r = RET_ERROR_MISSING;
 		}
-		if( RET_WAS_ERROR(r) ) {
+		if (RET_WAS_ERROR(r)) {
 			// TODO: instead store in downloaditem?
 			return r;
 		}
 		checksums = read_checksums;
 	}
-	assert( checksums != NULL );
+	assert (checksums != NULL);
 
-	if( !checksums_check(d->checksums, checksums, &improves) ) {
+	if (!checksums_check(d->checksums, checksums, &improves)) {
 		fprintf(stderr, "Wrong checksum during receive of '%s':\n",
 				uri);
 		checksums_printdifferences(stderr, d->checksums, checksums);
@@ -134,22 +134,22 @@ static retvalue downloaditem_callback(enum queue_action action, void *privdata,
 		// TODO: instead store in downloaditem?
 		return RET_ERROR_WRONG_MD5;
 	}
-	if( improves ) {
+	if (improves) {
 		r = checksums_combine(&d->checksums, checksums, NULL);
 		checksums_free(read_checksums);
-		if( RET_WAS_ERROR(r) )
+		if (RET_WAS_ERROR(r))
 			return r;
 	} else
 		checksums_free(read_checksums);
 
-	if( global.showdownloadpercent > 0 ) {
+	if (global.showdownloadpercent > 0) {
 		unsigned int percent;
 
 		cache->size_done += checksums_getfilesize(d->checksums);
 
 		percent = (100 * cache->size_done) / cache->size_todo;
-		if( global.showdownloadpercent > 1
-				|| percent > cache->last_percent ) {
+		if (global.showdownloadpercent > 1
+				|| percent > cache->last_percent) {
 			unsigned long long all = cache->size_done;
 			int kb, mb, gb, tb, b, groups = 0;
 
@@ -165,29 +165,29 @@ static retvalue downloaditem_callback(enum queue_action action, void *privdata,
 			gb = all & 1023;
 			all = all >> 10;
 			tb = all;
-			if( tb != 0 ) {
+			if (tb != 0) {
 				printf("%dT ", tb);
 				groups++;
 			}
-			if( groups < 2 && (groups > 0 || gb != 0 ) ) {
+			if (groups < 2 && (groups > 0 || gb != 0)) {
 				printf("%dG ", gb);
 				groups++;
 			}
-			if( groups < 2 && (groups > 0 || mb != 0 ) ) {
+			if (groups < 2 && (groups > 0 || mb != 0)) {
 				printf("%dM ", mb);
 				groups++;
 			}
-			if( groups < 2 && (groups > 0 || kb != 0 ) ) {
+			if (groups < 2 && (groups > 0 || kb != 0)) {
 				printf("%dK ", kb);
 				groups++;
 			}
-			if( groups < 2 && (groups > 0 || b != 0 ) )
+			if (groups < 2 && (groups > 0 || b != 0))
 				printf("%d ", b);
 			puts("bytes");
 		}
 	}
-	r = files_add_checksums(database, d->filekey, d->checksums);
-	if( RET_WAS_ERROR(r) )
+	r = files_add_checksums(d->filekey, d->checksums);
+	if (RET_WAS_ERROR(r))
 		return r;
 	d->done = true;
 	return RET_OK;
@@ -203,12 +203,12 @@ static retvalue downloaditem_callback(enum queue_action action, void *privdata,
 	*h = &list->items;
 	*p = NULL;
 	item = list->items;
-	while( item != NULL ) {
+	while (item != NULL) {
 		*p = item;
 		c = strcmp(filekey, item->filekey);
-		if( c == 0 )
+		if (c == 0)
 			return item;
-		else if( c < 0 ) {
+		else if (c < 0) {
 			*h = &item->left;
 			item = item->left;
 		} else {
@@ -222,66 +222,65 @@ static retvalue downloaditem_callback(enum queue_action action, void *privdata,
 /* queue a new file to be downloaded:
  * results in RET_ERROR_WRONG_MD5, if someone else already asked
  * for the same destination with other md5sum created. */
-retvalue downloadcache_add(struct downloadcache *cache, struct database *database, struct aptmethod *method, const char *orig, const char *filekey, const struct checksums *checksums) {
+retvalue downloadcache_add(struct downloadcache *cache, struct aptmethod *method, const char *orig, const char *filekey, const struct checksums *checksums) {
 
 	struct downloaditem *i;
-	struct downloaditem *item,**h,*parent;
+	struct downloaditem *item, **h, *parent;
 	char *fullfilename;
 	retvalue r;
 
-	assert( cache != NULL && method != NULL );
-	r = files_expect(database, filekey, checksums, false);
-	if( r != RET_NOTHING )
+	assert (cache != NULL && method != NULL);
+	r = files_expect(filekey, checksums, false);
+	if (r != RET_NOTHING)
 		return r;
 
-	i = searchforitem(cache,filekey,&parent,&h);
-	if( i != NULL ) {
+	i = searchforitem(cache, filekey, &parent, &h);
+	if (i != NULL) {
 		bool improves;
 
-		assert( i->filekey != NULL );
-		if( !checksums_check(i->checksums, checksums, &improves) ) {
+		assert (i->filekey != NULL);
+		if (!checksums_check(i->checksums, checksums, &improves)) {
 			fprintf(stderr,
 "ERROR: Same file is requested with conflicting checksums:\n");
 			checksums_printdifferences(stderr,
 					i->checksums, checksums);
 			return RET_ERROR_WRONG_MD5;
 		}
-		if( improves ) {
+		if (improves) {
 			r = checksums_combine(&i->checksums,
 					checksums, NULL);
-			if( RET_WAS_ERROR(r) )
+			if (RET_WAS_ERROR(r))
 				return r;
 		}
 		return RET_NOTHING;
 	}
-	item = calloc(1, sizeof(struct downloaditem));
-	if( FAILEDTOALLOC(item) )
+	item = zNEW(struct downloaditem);
+	if (FAILEDTOALLOC(item))
 		return RET_ERROR_OOM;
 
 	item->done = false;
 	item->filekey = strdup(filekey);
 	item->checksums = checksums_dup(checksums);
-	if( FAILEDTOALLOC(item->filekey) || FAILEDTOALLOC(item->checksums) ) {
+	if (FAILEDTOALLOC(item->filekey) || FAILEDTOALLOC(item->checksums)) {
 		freeitem(item);
 		return RET_ERROR_OOM;
 	}
 
 	fullfilename = files_calcfullfilename(filekey);
-	if( FAILEDTOALLOC(fullfilename) ) {
+	if (FAILEDTOALLOC(fullfilename)) {
 		freeitem(item);
 		return RET_ERROR_OOM;
 	}
 	(void)dirs_make_parent(fullfilename);
 	r = space_needed(cache->devices, fullfilename, checksums);
-	if( RET_WAS_ERROR(r) ) {
+	if (RET_WAS_ERROR(r)) {
 		free(fullfilename);
 		freeitem(item);
 		return r;
 	}
-	cache->database = database;
 	r = aptmethod_enqueue(method, orig, fullfilename,
 			downloaditem_callback, item, cache);
-	if( RET_WAS_ERROR(r) ) {
+	if (RET_WAS_ERROR(r)) {
 		freeitem(item);
 		return r;
 	}
@@ -296,25 +295,21 @@ retvalue downloadcache_add(struct downloadcache *cache, struct database *databas
 }
 
 /* some as above, only for more files... */
-retvalue downloadcache_addfiles(struct downloadcache *cache,
-		struct database *database,
-		struct aptmethod *method,
-		const struct checksumsarray *origfiles,
-		const struct strlist *filekeys) {
-	retvalue result,r;
+retvalue downloadcache_addfiles(struct downloadcache *cache, struct aptmethod *method, const struct checksumsarray *origfiles, const struct strlist *filekeys) {
+	retvalue result, r;
 	int i;
 
-	assert( origfiles != NULL && filekeys != NULL
-		&& origfiles->names.count == filekeys->count );
+	assert (origfiles != NULL && filekeys != NULL
+		&& origfiles->names.count == filekeys->count);
 
 	result = RET_NOTHING;
 
-	for( i = 0 ; i < filekeys->count ; i++ ) {
-		r = downloadcache_add(cache, database, method,
+	for (i = 0 ; i < filekeys->count ; i++) {
+		r = downloadcache_add(cache, method,
 			origfiles->names.values[i],
 			filekeys->values[i],
 			origfiles->checksums[i]);
-		RET_UPDATE(result,r);
+		RET_UPDATE(result, r);
 	}
 	return result;
 }
