@@ -23,7 +23,6 @@
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
-#include <malloc.h>
 #include "error.h"
 #include "mprintf.h"
 #include "strlist.h"
@@ -721,7 +720,7 @@ static retvalue parse_condition(const struct filebeingparsed *fbp, int column, c
 	const char *p = *pp;
 	struct upload_condition *fallback, *last, *or_scope;
 
-	memset(condition, 0, sizeof(struct upload_condition));
+	setzero(struct upload_condition, condition);
 
 	/* allocate a new fallback-node:
 	 * (this one is used to make it easier to concatenate those decision
@@ -942,7 +941,7 @@ static retvalue parse_condition(const struct filebeingparsed *fbp, int column, c
 			errorcol(fbp, column + (int)(p - *pp),
 "'by','and' or 'or' keyword expected!");
 			uploadpermission_release(condition);
-			memset(condition, 0, sizeof(struct upload_condition));
+			setzero(struct upload_condition, condition);
 			return RET_ERROR;
 		}
 		while (*p != '\0' && xisspace(*p))
@@ -956,7 +955,7 @@ static void condition_add(struct upload_condition *permissions, struct upload_co
 	if (permissions->next == NULL) {
 		/* first condition, as no fallback yet allocated */
 		*permissions = *c;
-		memset(c, 0, sizeof(struct upload_condition));
+		setzero(struct upload_condition, c);
 	} else {
 		struct upload_condition *last;
 
@@ -971,7 +970,7 @@ static void condition_add(struct upload_condition *permissions, struct upload_co
 		assert(!last->accept_if_true);
 
 		*last = *c;
-		memset(c, 0, sizeof(struct upload_condition));
+		setzero(struct upload_condition, c);
 	}
 }
 
